import React, { useEffect, useState } from 'react'
import { ChevronDownIcon, ChevronUpIcon, PencilIcon, TrashIcon, XIcon, SearchIcon } from '@heroicons/react/outline'
import { IconButton, Pagination, Backdrop, CircularProgress } from '@mui/material'
import useSWR from 'swr'
import ProtectedUi from './ProtectedUi'
import useAxios from 'src/hooks/useAxios'
import { DebounceInput } from 'react-debounce-input'
import { useRouter } from 'next/router'
import TableLoading from './TableLoading'
import useErrors from 'src/hooks/useErrors'
import swal from 'sweetalert'
import { useRecoilState } from 'recoil'
import { messageAtom } from 'src/lib/atoms/messageAtom'

const DataTable = ({
  url,
  urlKey,
  columns,
  editRole,
  deleteRole,
  defaultSortField = 'id',
  actions = true,
  editAction = true,
  deleteAction = true,
  otherActions = [],
  defaultLimit = 10,
  urlParams,
  trBorder = true,
  baseURL = null,
  shouldRevalidate = false,
  onRevalidateCompleted,
  deleteConfirmationTitle = 'Are you sure?',
  deleteDescription = null,
  editType = 'link',
  editUrl = '',
  deleteUrl = '',
  returnId
}) => {
  const [sortField, setSortField] = useState(defaultSortField)
  const [sortDirection, setSortDirection] = useState('asc')
  const [search, setSearch] = useState('')
  const [limit, setLimit] = useState(defaultLimit)
  const [page, setPage] = useState(1)
  const router = useRouter()
  const [totalPage, setTotalPage] = useState(0)
  const [deleteLoading, setDeleteLoading] = useState(false)
  const request = useAxios(baseURL)
  const { errorHandler } = useErrors()
  const setMessage = useRecoilState(messageAtom)

  const fetchData = (key, page, sort, direction, limit, search) => {
    return request
      .get()
      .then(res => res.data)
      .catch(err => errorHandler(err))
  }

  const { data, revalidate } = useSWR([!urlKey ? url : urlKey, page, sortField, sortDirection, limit, search], () =>
    request
      .get(
        `${url}/?search=${search}&sort=${sortField}&direction=${sortDirection}&page=${page}&limit=${limit}${
          urlParams ? `&${urlParams}` : ''
        }`
      )
      .then(res => res.data)
      .catch(err => console.log(err))
  )

  const sortIndicator = column => {
    if (sortField === column) {
      return sortDirection === 'asc' ? <ChevronDownIcon className='h-3 pr-3' /> : <ChevronUpIcon className='h-3 pr-3' />
    }

    return ''
  }

  const changeSort = column => {
    setSortField(column)
    setSortDirection(prev => (prev === 'asc' ? 'desc' : 'asc'))
  }

  const handlePage = (event, value) => {
    setPage(value)
    router.push({
      pathname: router.pathname,
      query: { ...router.query, page: value }
    })
  }
  useEffect(() => {
    if (data) {
      let p_number = data.total / limit
      setTotalPage(data.total / limit > parseInt(data.total / limit) ? parseInt(p_number) + 1 : parseInt(p_number))
    }
  }, [data])

  const resolve = (path, obj) => {
    return path.split('.').reduce(function (prev, curr) {
      return prev ? prev[curr] : null
    }, obj || self)
  }

  const td = (value, type) => {
    if (typeof type === 'undefined') {
      return value
    } else {
      switch (type) {
        case 'boolean':
          return value ? 'Active' : 'Inactive'
        case 'yes/no':
          return value ? 'Yes' : 'No'
        case 'nullableInt':
          return value !== null ? value : 0
        case 'nullableStr':
          return value !== null ? value : ''
      }
    }
  }

  useEffect(() => {
    if (shouldRevalidate) {
      revalidate().then(() => {
        onRevalidateCompleted()
      })
    }
  }, [shouldRevalidate])

  const deleteRow = id => {
    swal({
      icon: 'warning',
      title: deleteConfirmationTitle,
      text: deleteDescription,
      buttons: true
    }).then(res => {
      if (res) {
        setDeleteLoading(true)
        let path = null
        if (deleteUrl) {
          path = deleteUrl
        } else {
          path = url
        }
        request
          .delete(path + '/' + id)
          .then(res => {
            setDeleteLoading(false)
            if (res?.data?.status) {
              revalidate()
              setMessage[1](res?.data?.message)
            } else {
              swal({
                icon: 'error',
                title: 'Something went wrong',
                text: res.data?.message
              })
            }
          })
          .catch(err => {
            errorHandler(err)
            setDeleteLoading(false)
          })
      }
    })
  }

  return (
    <div>
      <div>
        <div className='flex items-center justify-between'>
          <div className={'flex w-full items-center rounded-full border py-1 px-3 border-[#999] max-w-fit'}>
            <div className={'flex-1'}>
              <DebounceInput
                className={
                  'w-full border-none border-transparent bg-transparent text-xs outline-none ring-0 focus:border-transparent focus:ring-0'
                }
                placeholder={'Search'}
                debounceTimeout={500}
                value={search}
                onChange={e => {
                  setSearch(e.target.value)
                  setPage(1)
                  router.push({
                    pathname: router.pathname,
                    query: {
                      ...router.query,
                      search: e.target.value
                    }
                  })
                }}
              />
            </div>
            <div className={'flex items-center'}>
              {search && <XIcon className={'mr-2 h-4 cursor-pointer'} onClick={() => setSearch('')} />}
              <SearchIcon className={'mr-2 h-4'} />
            </div>
          </div>
          <div className={'mr-5 mt-5 flex content-center items-center justify-center space-x-2 lg:mt-0'}>
            <label htmlFor='select'>Total record per page</label>
            <select
              id='select'
              className='border'
              onChange={e => {
                setPage(1)
                setLimit(e.target.value)
                router.push({
                  pathname: router.pathname,
                  query: {
                    ...router.query,
                    limit: e.target.value
                  }
                })
              }}
            >
              <option value={10}>10</option>
              <option value={20}>20</option>
              <option value={50}>50</option>
              <option value={100}>100</option>
              <option value={200}>200</option>
            </select>
          </div>
        </div>

        {data ? (
          <div className={'overflow-x-auto'}>
            <table className={`w-full text-left mt-4`}>
              <thead className='border-b py-3 bg-gray-100'>
                <tr>
                  {columns?.map(column => (
                    <th
                      key={column?.name}
                      className={`whitespace-nowrap mx-1 px-4`}
                      onClick={() => column?.sort && changeSort(column?.key)}
                    >
                      <div className={`flex items-center space-x-2 ${column?.sort && 'cursor-pointer'}`}>
                        <div className='flex-grow'>{column?.name}</div>
                        {column?.sort && sortIndicator(column?.key)}
                      </div>
                    </th>
                  ))}
                  {actions && <th className='px-4'>Actions</th>}
                </tr>
              </thead>
              <tbody>
                {data?.total < 1 ? (
                  <tr>
                    <td className='text-red-500 text-center py-4' colSpan={columns?.length + 1}>
                      No record found
                    </td>
                  </tr>
                ) : (
                  data?.data?.map((item, index) => (
                    <tr key={index} className={`${trBorder && 'border-b'}`}>
                      {columns?.map((column, index) => (
                        <td key={index} className='px-4'>
                          <span>
                            {td(
                              resolve(
                                column?.key,

                                // {(typeof item == 'boolean') ? (item)
                                item
                              ),
                              column?.type
                            )}
                          </span>
                        </td>
                      ))}
                      {actions && (
                        <td className='px-6 flex items-center'>
                          {editAction && (
                            <IconButton
                              onClick={() =>
                                editType === 'link' ? router.push(`${editUrl}${item.id}`) : returnId(item.id)
                              }
                              size={`small`}
                            >
                              <PencilIcon className='h-4 text-blue-500' />
                            </IconButton>
                          )}
                          {deleteAction && (
                            <IconButton onClick={() => deleteRow(item?.id)} size={`small`}>
                              <TrashIcon className='h-4 text-red-500' />
                            </IconButton>
                          )}
                          {otherActions && (
                            <>
                              {otherActions.map((actions, index) => (
                                <ProtectedUi key={`actions${index}`} role={actions.role}>
                                  <IconButton
                                    onClick={() =>
                                      typeof actions?.full_url !== 'undefined'
                                        ? window.open(
                                            actions?.route +
                                              `${typeof actions?.has_param !== 'undefined' ? '&' : '?'}id=` +
                                              item?.id,
                                            '_blank'
                                          )
                                        : typeof actions?.custom_id === 'undefined'
                                        ? router.push(
                                            actions.route +
                                              `&${typeof actions.id_param === 'undefined' ? 'id' : actions.id_param}=` +
                                              item.id
                                          )
                                        : router.push(
                                            actions.route +
                                              `&${typeof actions.id_param === 'undefined' ? 'id' : actions.id_param}=` +
                                              resolve(actions.custom_id, item)
                                          )
                                    }
                                    ripple={'dark'}
                                    size={'small'}
                                  >
                                    {actions.icon}
                                  </IconButton>
                                </ProtectedUi>
                              ))}
                            </>
                          )}
                        </td>
                      )}
                    </tr>
                  ))
                )}
              </tbody>
            </table>
            <div className={'mt-4 flex items-center pb-3'}>
              <div className={'flex-1'}>
                <Pagination page={page} onChange={handlePage} count={totalPage} color='primary' />
              </div>
              <div>
                <p className={'mr-12 text-sm'}>
                  <b>Total results: ({data?.total})</b>
                </p>
              </div>
            </div>
          </div>
        ) : (
          <TableLoading />
        )}
      </div>
      <Backdrop
        sx={{
          color: '#fff',
          zIndex: theme => theme.zIndex.drawer + 1
        }}
        open={deleteLoading}
      >
        <CircularProgress color='inherit' />
      </Backdrop>
    </div>
  )
}

export default DataTable
